package com.aliyun.chatbot.channel.feishu;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.TimeUnit;

import com.alibaba.fastjson2.JSON;

import com.aliyun.chatbot.channel.utils.OkHttpUtils;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

/**
 * @author kelin
 * @date 2024/6/4
 */
@Slf4j
public class FeishuAccessTokenClient {
    private final String appId;
    private final String appSecret;

    private static final LoadingCache<TokenRequest, String> CACHE = CacheBuilder.newBuilder()
        .maximumSize(10)
        .expireAfterWrite(100, TimeUnit.MINUTES)
        .build(new CacheLoader<TokenRequest, String>() {
            @Override
            public String load(TokenRequest key) throws Exception {
                return getAccessToken(key);
            }
        });

    public FeishuAccessTokenClient(String appId, String appSecret) {
        this.appId = appId;
        this.appSecret = appSecret;
    }

    public String getAccessToken() throws Exception {
        return CACHE.get(new TokenRequest(appId, appSecret));
    }

    /**
     * 获取企业内部应用的accessToken
     * https://open.feishu.cn/document/server-docs/authentication-management/access-token/tenant_access_token_internal
     */
    private static String getAccessToken(TokenRequest key) throws Exception {
        Map<String, String> p = new HashMap<>();
        p.put("app_id", key.getAppId());
        p.put("app_secret", key.getAppSecret());
        String s = OkHttpUtils.postJsonParams("https://open.feishu.cn/open-apis/auth/v3/tenant_access_token/internal",
            JSON.toJSONString(p));
        log.info("feishu accessToken:{}", s);
        return JSON.parseObject(s).getString("tenant_access_token");
    }

    @Data
    public static class TokenRequest {
        private final String appId;
        private final String appSecret;
    }

}
